@@ -253,15 +253,8 @@ def test_from_tensors(weights_factory):
 
 
 def test_overloaded_operators(data1):
-    a = data1 * 5.0
-    example_data1 = data1.value().numpy()
-    np.testing.assert_array_equal(5 * example_data1, a.numpy())
-    np.testing.assert_array_equal(example_data1, data1.numpy())
-    data_squared = data1 * data1
-    np.testing.assert_allclose(example_data1**2, data_squared.numpy(), rtol=1e-8)
-    np.testing.assert_allclose(
-        np.log(example_data1), tf.math.log(data1).numpy(), rtol=1e-8
-    )
+    with pytest.raises(TypeError):
+        a = data1 * 5.0
 
 
 def test_sort_by_obs(data1, obs3d):
@@ -286,6 +279,24 @@ def test_sort_by_obs(data1, obs3d):
     np.testing.assert_array_equal(example_data1, data1.value().numpy())
 
 
+def test_data_axis_access(obs3d, data1):
+    import zfit.z.numpy as znp
+
+    true_mapping = {
+        obs: znp.reshape(arr, (-1, 1))
+        for obs, arr in zip(obs3d, data1.value().numpy().T)
+    }
+    for obs in obs3d:
+        np.testing.assert_allclose(data1.value(obs), true_mapping[obs][:, 0])
+        np.testing.assert_allclose(data1.value([obs]), true_mapping[obs])
+        np.testing.assert_allclose(data1[obs], true_mapping[obs][:, 0])
+        np.testing.assert_allclose(data1[[obs]], true_mapping[obs])
+    obs2d = [obs3d[2], obs3d[1]]
+    array2d = znp.concatenate([true_mapping[obs] for obs in obs2d], axis=1)
+    np.testing.assert_allclose(data1[obs2d], array2d)
+    np.testing.assert_allclose(data1.value(obs2d), array2d)
+
+
 def test_subdata(obs3d, data1):
     new_obs = (obs3d[0], obs3d[1])
     new_array = copy.deepcopy(data1.value().numpy())[:, np.array((0, 1))]
@@ -387,14 +398,14 @@ def test_data_hashing(space2d):
     assert oldhashint != testhashpdf.lasthash
     assert data1.hashint == testhashpdf.lasthash
 
-    zfit.run.set_graph_mode(
+    with zfit.run.set_graph_mode(
         True
-    )  # meaning integration is now done in graph and has "None"
-    oldhashint = data1.hashint
-    data1.set_weights(np.random.uniform(size=data1.nevents))
-    testhashpdf.pdf(data1)
-    assert oldhashint != testhashpdf.lasthash
-    assert None == testhashpdf.lasthash
+    ):  # meaning integration is now done in graph and has "None"
+        oldhashint = data1.hashint
+        data1.set_weights(np.random.uniform(size=data1.nevents))
+        testhashpdf.pdf(data1)
+        assert oldhashint != testhashpdf.lasthash
+        assert None == testhashpdf.lasthash
 
 
 def test_hashing_resample(space2d):
