@@ -262,7 +262,7 @@ def test_binned_template_pdf_bbfull(TemplateLikePDF):
     )
     # for i in progressbar.progressbar(range(1000000)):
     loss.value()
-    loss.gradients()
+    loss.gradient()
     minimizer = zfit.minimize.Minuit(gradient=False, mode=0)
     # minimizer = zfit.minimize.ScipyTrustConstrV1(verbosity=8)
     minimizer.minimize(loss)
