@@ -1,4 +1,4 @@
-#  Copyright (c) 2021 zfit
+#  Copyright (c) 2022 zfit
 import math
 
 import pytest
@@ -9,9 +9,10 @@ import zfit
 @pytest.mark.flaky(2)
 def test_fail_on_nan_strategy():
     from zfit.minimizers.strategy import ToyStrategyFail
-    sigma = zfit.Parameter("sigma", 2.)
+
+    sigma = zfit.Parameter("sigma", 2.0)
     obs = zfit.Space("obs1", limits=(-4, 5))
-    gauss = zfit.pdf.Gauss(1., sigma, obs=obs)
+    gauss = zfit.pdf.Gauss(1.0, sigma, obs=obs)
 
     sampler = gauss.create_sampler(3000)
     sampler.set_data_range(obs)
@@ -26,8 +27,8 @@ def test_fail_on_nan_strategy():
     sigma.set_value(math.inf)
     fitresult2 = minimizer.minimize(nll)
     assert not fitresult2.converged
-    assert fitresult2.edm == -999
-    assert fitresult2.fmin == -999
+    assert fitresult2.edm is None
+    assert fitresult2.fmin is None
 
 
 def minimizers():
@@ -44,7 +45,7 @@ def minimizers():
 minimizers = sorted(minimizers(), key=lambda val: repr(val))
 
 
-@pytest.mark.parametrize('minimizer_cls', minimizers)
+@pytest.mark.parametrize("minimizer_cls", minimizers)
 def test_callback(minimizer_cls):
     from zfit.minimizers.strategy import PushbackStrategy
 
@@ -56,7 +57,7 @@ def test_callback(minimizer_cls):
             raise MyError
 
     minimizer = zfit.minimize.ScipySLSQPV1(strategy=MyStrategy)
-    loss = lambda *args, **kwargs: 42.
+    loss = lambda *args, **kwargs: 42.0
     loss.errordef = 0.5
     with pytest.raises(MyError):
         minimizer.minimize(loss, [1, 3])
