@@ -8,7 +8,7 @@ import tensorflow_probability as tfp
 
 import zfit.z.numpy as znp
 from zfit import z
-from zfit.core.interfaces import ZfitData, ZfitRectBinning
+from zfit.core.interfaces import ZfitData, ZfitRectBinning, ZfitSpace
 from zfit.util.ztyping import XTypeInput
 
 
@@ -51,6 +51,16 @@ def histogramdd(sample, bins=10, range=None, weights=None, density=None):
 
 
 def unbinned_to_hist_eager(values, edges, weights=None):
+    """Convert an unbinned dataset to a binned dataset in eager mode.
+
+    Args:
+        values: Unbinned dataset to convert.
+        edges: Edges of the bins.
+        weights: Event weights.
+
+    Returns:
+        binned_data: Binned dataset.
+    """
     if weights is not None and weights.shape == () and None in weights:
         weights = None
     binning = [
@@ -65,10 +75,30 @@ def unbinned_to_hist_eager(values, edges, weights=None):
 
 
 def unbinned_to_binned(data, space, binned_class=None):
+    """Convert an unbinned dataset to a binned dataset.
+
+    Args:
+        data: Unbinned dataset to convert.
+        space: Space to bin the data in.
+        binned_class: Class to use for the binned dataset. Defaults to `BinnedData`.
+
+    Returns:
+        binned_data: Binned dataset of type `binned_class`.
+    """
     if binned_class is None:
         from zfit._data.binneddatav1 import BinnedData
 
         binned_class = BinnedData
+    if not isinstance(space, ZfitSpace):
+        try:
+            space = data.space.with_binning(space)
+        except Exception as error:
+            raise ValueError(
+                f"The space provided is not a valid space for the data. "
+                f"Either provide a valid space or a binning. "
+                f"The error was: {error}"
+            ) from error
+
     values = data.value()
     weights = data.weights
     if weights is not None:
@@ -84,6 +114,16 @@ def unbinned_to_binned(data, space, binned_class=None):
 
 
 def unbinned_to_binindex(data, space, flow=False):
+    """Calculate the bin index of each data point.
+
+    Args:
+        data: Data to calculate the bin index for.
+        space: Defines the binning.
+        flow: Whether to include the underflow and overflow bins.
+
+    Returns:
+        binindex: Tensor with shape (ndata, n_obs) holding the bin index of each data point.
+    """
     if flow:
         warnings.warn(
             "Flow currently not fully supported. Values outside the edges are all 0."
