@@ -7,6 +7,8 @@ from typing import TYPE_CHECKING, Union
 import xxhash
 from tensorflow.python.util.deprecation import deprecated_args, deprecated
 
+from .parameter import set_values
+
 if TYPE_CHECKING:
     import zfit
 
@@ -29,7 +31,7 @@ from .. import z
 from ..settings import ztypes, run
 from ..util import ztyping
 from ..util.cache import GraphCachable, invalidate_graph
-from ..util.container import convert_to_container
+from ..util.container import convert_to_container, is_container
 from ..util.exception import (
     ObsIncompatibleError,
     ShapeIncompatibleError,
@@ -45,9 +47,7 @@ from .space import Space, convert_to_space
 
 
 # TODO: make cut only once, then remember
-class Data(
-    ZfitUnbinnedData, BaseDimensional, BaseObject, OverloadableMixin, GraphCachable
-):
+class Data(ZfitUnbinnedData, BaseDimensional, BaseObject, GraphCachable):
     BATCH_SIZE = 1000000  # 1 mio
 
     def __init__(
@@ -94,7 +94,7 @@ class Data(
         self.dataset = dataset.batch(100_000_000)
         self._name = name
 
-        self.set_weights(weights=weights)
+        self._set_weights(weights=weights)
         self._hashint = None
         self._update_hash()
 
@@ -156,6 +156,7 @@ class Data(
 
     @property
     def weights(self):
+        """Get the weights of the data."""
         # TODO: refactor below more general, when to apply a cut?
         if self.data_range.has_limits and self.has_weights:
             raw_values = self._value_internal(obs=self.data_range.obs, filter=False)
@@ -173,21 +174,27 @@ class Data(
         Args:
             weights:
         """
-        if weights is not None:
-            weights = z.convert_to_tensor(weights)
-            weights = z.to_real(weights)
-            if weights.shape.ndims != 1:
-                raise ShapeIncompatibleError("Weights have to be 1-Dim objects.")
+
+        # weights = self._set_weights(weights)
 
         def setter(value):
-            self._weights = value
-            self._update_hash()
+            self._set_weights(value)
 
         def getter():
             return self.weights
 
         return TemporarilySet(value=weights, getter=getter, setter=setter)
 
+    def _set_weights(self, weights):
+        if weights is not None:
+            weights = z.convert_to_tensor(weights)
+            weights = z.to_real(weights)
+            if weights.shape.ndims != 1:
+                raise ShapeIncompatibleError("Weights have to be 1-Dim objects.")
+        self._weights = weights
+        self._update_hash()
+        return weights
+
     @property
     def space(self) -> ZfitSpace:
         return self._space
@@ -435,6 +442,14 @@ class Data(
                 self._hashint = None
 
     def with_obs(self, obs):
+        """Create a new ``Data`` with a subset of the data using the *obs*.
+
+        Args:
+            obs: Observables to return. Has to be a subset of the original observables.
+
+        Returns:
+            ``zfit.Data``: A new ``Data`` object containing the subset of the data.
+        """
         values = self.value(obs)
         return type(self).from_tensor(
             obs=self.space, tensor=values, weights=self.weights, name=self.name
@@ -469,7 +484,19 @@ class Data(
         return z.unstack_x(self.value(obs=obs))
 
     def value(self, obs: ztyping.ObsTypeInput = None):
-        return znp.asarray(self._value_internal(obs=obs))
+        """Return the data as a numpy-like object in ``obs`` order.
+
+        Args:
+            obs: Observables to return. If ``None``, all observables are returned. Can be a subset of the original
+                observables. If a string is given, a 1-D array is returned with shape (nevents,). If a list of strings
+                or a ``zfit.Space`` is given, a 2-D array is returned with shape (nevents, nobs).
+
+        Returns:
+        """
+        out = znp.asarray(self._value_internal(obs=obs))
+        if isinstance(obs, str):
+            out = znp.squeeze(out, axis=-1)
+        return out
 
     def numpy(self):
         return self.value().numpy()
@@ -621,6 +648,15 @@ class Data(
 
         return BinnedData.from_unbinned(space=space, data=self)
 
+    def __getitem__(self, item):
+        return getitem_obs(self, item)
+
+
+def getitem_obs(self, item):
+    if not isinstance(item, str):
+        item = convert_to_obs_str(item)
+    return self.value(item)
+
 
 class SampleData(Data):
     _cache_counting = 0
@@ -796,12 +832,9 @@ class Sampler(Data):
         if param_values is not None:
             temp_param_values.update(param_values)
 
-        with ExitStack() as stack:
-
-            _ = [
-                stack.enter_context(param.set_value(val))
-                for param, val in temp_param_values.items()
-            ]
+        with set_values(
+            list(temp_param_values.keys()), list(temp_param_values.values())
+        ):
 
             # if not (n and self._initial_resampled):  # we want to load and make sure that it's initialized
             #     # means it's handled inside the function
@@ -824,7 +857,7 @@ class Sampler(Data):
         return f"<Sampler: {self.name} obs={self.obs}>"
 
 
-register_tensor_conversion(Data, name="Data", overload_operators=True)
+# register_tensor_conversion(Data, name="Data", overload_operators=True)
 
 
 class LightDataset:
