@@ -1,7 +1,11 @@
+#  Copyright (c) 2022 zfit
+
+from __future__ import annotations
+
 #  Copyright (c) 2020 zfit
 import abc
 import itertools
-from typing import Iterable
+from collections.abc import Iterable
 
 from ordered_set import OrderedSet
 
@@ -19,7 +23,7 @@ class BaseDependentsMixin(ZfitDependenciesMixin):
         """Return a set of all independent :py:class:`~zfit.Parameter` that this object depends on.
 
         Args:
-            only_floating: If `True`, only return floating :py:class:`~zfit.Parameter`
+            only_floating: If ``True``, only return floating :py:class:`~zfit.Parameter`
         """
         dependencies = self._get_dependencies()
         if only_floating:
