@@ -70,7 +70,7 @@ def _unbinned_nll_tf(
         fit_range:
 
     Returns:
-        The unbinned nll
+        The unbinned nll value as a scalar
     """
 
     if is_container(model):
@@ -78,13 +78,8 @@ def _unbinned_nll_tf(
             _unbinned_nll_tf(model=p, data=d, fit_range=r, log_offset=log_offset)
             for p, d, r in zip(model, data, fit_range)
         ]
-        # nlls_total = [nll.total for nll in nlls]
-        # nlls_correction = [nll.correction for nll in nlls]
-        # nlls_total_summed = znp.sum(input_tensor=nlls_total, axis=0)
         nlls_summed = znp.sum(nlls, axis=0)
 
-        # nlls_correction_summed = znp.sum(input_tensor=nlls_correction, axis=0)
-        # nll_finished = (nlls_total_summed, nlls_correction_summed)
         nll_finished = nlls_summed
     else:
         if fit_range is not None:
@@ -452,7 +447,6 @@ class BaseLoss(ZfitLoss, BaseNumeric):
         # value = value_substracted[0] - value_substracted[1]
 
     def _value(self, model, data, fit_range, constraints, log_offset):
-        # try:
         return self._loss_func(
             model=model,
             data=data,
@@ -461,9 +455,6 @@ class BaseLoss(ZfitLoss, BaseNumeric):
             log_offset=log_offset,
         )
 
-    # except NotImplementedError as error:
-    #     raise NotImplementedError(f"_loss_func not properly defined! error {error}") from error
-
     def __add__(self, other):
         if not isinstance(other, BaseLoss):
             raise TypeError(
@@ -485,9 +476,10 @@ class BaseLoss(ZfitLoss, BaseNumeric):
         params = {p.name: p for p in params}
         return self._gradient(params=params, numgrad=numgrad)
 
-    @deprecated(None, "Use `gradient` instead.")
     def gradients(self, *args, **kwargs):
-        return self.gradient(*args, **kwargs)
+        raise BreakingAPIChangeError(
+            "`gradients` is deprecated, use `gradient` instead."
+        )
 
     @z.function(wraps="loss")
     def _gradient(self, params, numgrad):
@@ -507,9 +499,10 @@ class BaseLoss(ZfitLoss, BaseNumeric):
 
         return self._value_gradient(params=params, numgrad=numgrad)
 
-    @deprecated(None, "Use `value_gradient` instead.")
     def value_gradients(self, *args, **kwargs):
-        return self.value_gradient(*args, **kwargs)
+        raise BreakingAPIChangeError(
+            "`value_gradients` is deprecated, use `value_gradient` instead."
+        )
 
     @z.function(wraps="loss")
     def _value_gradient(self, params, numgrad=False):
@@ -536,9 +529,10 @@ class BaseLoss(ZfitLoss, BaseNumeric):
         vals = vals[0], z.convert_to_tensor(vals[1]), vals[2]
         return vals
 
-    @deprecated(None, "Use `value_gradient_hessian` instead.")
     def value_gradients_hessian(self, *args, **kwargs):
-        return self.value_gradient_hessian(*args, **kwargs)
+        raise BreakingAPIChangeError(
+            "`value_gradients_hessian` is deprecated, use `value_gradient_hessian` instead."
+        )
 
     @z.function(wraps="loss")
     def _value_gradient_hessian(self, params, hessian, numerical=False):
