@@ -1,22 +1,30 @@
-#  Copyright (c) 2021 zfit
+#  Copyright (c) 2022 zfit
+
+from __future__ import annotations
+
 import contextlib
-from typing import Callable, Optional, Tuple
+from collections.abc import Callable
 
 import numpy as np
 import tensorflow as tf
 import texttable as tt
 
+from ..util.container import convert_to_container
+from ..z import numpy as znp
+from .strategy import ZfitStrategy
 from ..core.interfaces import ZfitLoss
-from ..core.parameter import assign_values, set_values
+from ..core.parameter import assign_values, assign_values_jit
 from ..settings import run
 from ..util import ztyping
-from ..util.exception import (DerivativeCalculationError,
-                              MaximumIterationReached)
-from .strategy import ZfitStrategy
+from ..util.exception import DerivativeCalculationError, MaximumIterationReached
+
+
+def assign_values_func(params, values):
+    return assign_values_jit(params, znp.asarray(values))
 
 
 def check_derivative_none_raise(values, params) -> None:
-    """Check if values contains `None` and raise a ``DerivativeCalculationError`` if so.
+    """Check if values contains ``None`` and raise a ``DerivativeCalculationError`` if so.
 
     Args:
         values: Values to check. Can be gradient or hessian
@@ -24,25 +32,29 @@ def check_derivative_none_raise(values, params) -> None:
     """
     if None in values:
         none_params = [p for p, grad in zip(params, values) if grad is None]
-        raise DerivativeCalculationError(f"The derivative of the following parameters is None: {none_params}."
-                                         f" This is usually caused by either the function not depending on the"
-                                         f" parameter (or not in a differentiable way) or by using pure Python"
-                                         f" code instead of TensorFlow.")
+        raise DerivativeCalculationError(
+            f"The derivative of the following parameters is None: {none_params}."
+            f" This is usually caused by either the function not depending on the"
+            f" parameter (or not in a differentiable way) or by using pure Python"
+            f" code instead of TensorFlow."
+        )
 
 
 class LossEval:
-
-    def __init__(self,
-                 loss: ZfitLoss,
-                 params: ztyping.ParamTypeInput,
-                 strategy: ZfitStrategy,
-                 do_print: bool,
-                 maxiter: int,
-                 grad_fn: Optional[Callable] = None,
-                 hesse_fn: Optional[Callable] = None):
+    def __init__(
+        self,
+        loss: ZfitLoss,
+        params: ztyping.ParamTypeInput,
+        strategy: ZfitStrategy,
+        do_print: bool,
+        maxiter: int,
+        grad_fn: Callable | None = None,
+        hesse_fn: Callable | None = None,
+        numpy_converter: Callable | None = None,
+    ):
         r"""Convenience wrapper for the evaluation of a loss with given parameters and strategy.
 
-        The methods `value`, `gradient` etc will raise a `MaximumIterationReached` error in case the maximum iterations
+        The methods ``value``, ``gradient`` etc will raise a ``MaximumIterationReached`` error in case the maximum iterations
         is reached.
 
         Args:
@@ -50,9 +62,14 @@ class LossEval:
             params: Parameters that the gradient and hessian will be derived to.
             strategy: A strategy to deal with NaNs and more.
             do_print: If the values should be printed nicely
-            maxiter: Maximum number of evaluations of the `value`, 'gradient` or `hessian`.
+            maxiter: Maximum number of evaluations of the ``value``, 'gradient`` or ``hessian``.
             grad_fn: Function that returns the gradient.
             hesse_fn: Function that returns the hessian matrix.
+            numpy_converter: Converter to a numpy-like format. For example, ``np.array`` will create
+                a *writable* numpy array, whereas ``np.asarray`` will create a *read-only* numpy array.
+                Useful if the return values should be numpy arrays (and not "numpy-like" objects). This is needed
+                if the function expects something *writeable* like a numpy array as other (JAX, TensorFlow) arrays
+                are not writeable. If None, no conversion is done and a "nmupy-like" object is returned.
         """
         super().__init__()
         self.maxiter = maxiter
@@ -68,8 +85,10 @@ class LossEval:
 
         self.hesse_fn = hesse_fn
         if grad_fn is not None:
+
             def value_gradients_fn(params):
                 return loss.value(), grad_fn(params)
+
         else:
             value_gradients_fn = self.loss.value_gradient
             grad_fn = self.loss.gradient
@@ -80,9 +99,10 @@ class LossEval:
         self.last_gradient = None
         self.last_hessian = None
         self.nan_counter = 0
-        self.params = params
+        self.params = convert_to_container(params)
         self.strategy = strategy
         self.do_print = do_print
+        self.numpy_converter = False if numpy_converter is None else numpy_converter
 
     @property
     def niter(self):
@@ -135,7 +155,7 @@ class LossEval:
         self._nhess_eval = value
         self._check_maxiter_reached()
 
-    def value_gradient(self, values: np.ndarray) -> Tuple[np.float64, np.ndarray]:
+    def value_gradient(self, values: np.ndarray) -> tuple[np.float64, np.ndarray]:
         """Calculate the value and gradient like :py:meth:`~ZfitLoss.value_gradients`.
 
         Args:
@@ -152,18 +172,18 @@ class LossEval:
             self.ngrad_eval += 1
 
         params = self.params
-        assign_values(params, values=values)
+        assign_values_func(params, values=values)
         is_nan = False
 
         try:
             loss_value, gradient = self.value_gradients_fn(params=params)
-            loss_value = run(loss_value)
-            gradient_values = np.array(run(gradient))
-            loss_value, gradient_values, _ = self.strategy.callback(value=loss_value,
-                                                                    gradient=gradient_values,
-                                                                    hessian=None,
-                                                                    params=params,
-                                                                    loss=self.loss)
+            loss_value, gradient_values, _ = self.strategy.callback(
+                value=loss_value,
+                gradient=gradient,
+                hessian=None,
+                params=params,
+                loss=self.loss,
+            )
         except Exception as error:
             loss_value = "invalid, error occured"
             gradient_values = ["invalid"] * len(params)
@@ -176,29 +196,33 @@ class LossEval:
 
             if self.do_print:
                 try:
-                    print_gradient(params, values, gradient=gradient_values, loss=loss_value)
+                    print_gradient(params, values, gradient=gradient, loss=loss_value)
                 except:
                     print("Cannot print loss value or gradient values.")
 
-        check_derivative_none_raise(gradient_values, params)
-        is_nan = is_nan or any(np.isnan(gradient_values)) or np.isnan(loss_value)
+        check_derivative_none_raise(gradient, params)
+        is_nan = is_nan or any(znp.isnan(gradient)) or znp.isnan(loss_value)
         if is_nan:
             self.nan_counter += 1
             info_values = {
-                'loss': loss_value,
-                'grad': gradient_values,
-                'old_loss': self.last_value,
-                'old_grad': self.last_gradient,
-                'nan_counter': self.nan_counter,
+                "loss": loss_value,
+                "grad": gradient,
+                "old_loss": self.last_value,
+                "old_grad": self.last_gradient,
+                "nan_counter": self.nan_counter,
             }
 
-            loss_value, gradient_values = self.strategy.minimize_nan(loss=self.loss, params=params,
-                                                                     values=info_values)
+            loss_value, gradient = self.strategy.minimize_nan(
+                loss=self.loss, params=params, values=info_values
+            )
         else:
             self.nan_counter = 0
             self.last_value = loss_value
-            self.last_gradient = gradient_values
-        return loss_value, gradient_values
+            self.last_gradient = gradient
+        if self.numpy_converter:
+            loss_value = float(loss_value)
+            gradient = self.numpy_converter(gradient)
+        return loss_value, gradient
 
     def value(self, values: np.ndarray) -> np.float64:
         """Calculate the value like :py:meth:`~ZfitLoss.value`.
@@ -215,17 +239,18 @@ class LossEval:
         if not self._ignoring_maxiter:
             self.nfunc_eval += 1
         params = self.params
-        assign_values(params, values=values)
+        assign_values_func(params, values=values)
         is_nan = False
 
         try:
             loss_value = self.loss.value()
-            loss_value = run(loss_value)
-            loss_value, _, _ = self.strategy.callback(value=loss_value,
-                                                      gradient=None,
-                                                      hessian=None,
-                                                      params=params,
-                                                      loss=self.loss)
+            loss_value, _, _ = self.strategy.callback(
+                value=loss_value,
+                gradient=None,
+                hessian=None,
+                params=params,
+                loss=self.loss,
+            )
         except Exception as error:
             loss_value = "invalid, error occured"
             if isinstance(error, tf.errors.InvalidArgumentError):
@@ -244,16 +269,19 @@ class LossEval:
         if is_nan:
             self.nan_counter += 1
             info_values = {
-                'loss': loss_value,
-                'old_loss': self.last_value,
-                'nan_counter': self.nan_counter,
+                "loss": loss_value,
+                "old_loss": self.last_value,
+                "nan_counter": self.nan_counter,
             }
 
-            loss_value, _ = self.strategy.minimize_nan(loss=self.loss, params=params,
-                                                       values=info_values)
+            loss_value, _ = self.strategy.minimize_nan(
+                loss=self.loss, params=params, values=info_values
+            )
         else:
             self.nan_counter = 0
             self.last_value = loss_value
+        if self.numpy_converter:
+            loss_value = float(loss_value)
         return loss_value
 
     def gradient(self, values: np.ndarray) -> np.ndarray:
@@ -271,19 +299,20 @@ class LossEval:
         if not self._ignoring_maxiter:
             self.ngrad_eval += 1
         params = self.params
-        assign_values(params, values=values)
+        assign_values_func(params, values=values)
         is_nan = False
 
         try:
             gradient = self.gradients_fn(params=params)
-            gradient_values = np.array(run(gradient))
-            _, gradient_values, _ = self.strategy.callback(value=None,
-                                                           gradient=gradient_values,
-                                                           hessian=None,
-                                                           params=params,
-                                                           loss=self.loss)
+            _, gradient, _ = self.strategy.callback(
+                value=None,
+                gradient=gradient,
+                hessian=None,
+                params=params,
+                loss=self.loss,
+            )
         except Exception as error:
-            gradient_values = ["invalid"] * len(params)
+            gradient = ["invalid"] * len(params)
             if isinstance(error, tf.errors.InvalidArgumentError):
                 is_nan = True
             else:
@@ -292,31 +321,34 @@ class LossEval:
         finally:
             if self.do_print:
                 try:
-                    print_gradient(params, values, gradient=gradient_values, loss=-999)
+                    print_gradient(params, values, gradient=gradient, loss=None)
                 except:
                     print("Cannot print loss value or gradient values.")
 
-        check_derivative_none_raise(gradient_values, params)
+        check_derivative_none_raise(gradient, params)
 
-        is_nan = is_nan or any(np.isnan(gradient_values))
+        is_nan = is_nan or znp.any(znp.isnan(gradient))
         if is_nan:
             self.nan_counter += 1
             info_values = {
-                'loss': -999,
-                'grad': gradient_values,
-                'old_loss': self.last_value,
-                'old_grad': self.last_gradient,
-                'nan_counter': self.nan_counter,
+                "loss": None,
+                "grad": gradient,
+                "old_loss": self.last_value,
+                "old_grad": self.last_gradient,
+                "nan_counter": self.nan_counter,
             }
 
-            _, gradient_values = self.strategy.minimize_nan(loss=self.loss, params=params,
-                                                            values=info_values)
+            _, gradient = self.strategy.minimize_nan(
+                loss=self.loss, params=params, values=info_values
+            )
         else:
             self.nan_counter = 0
-            self.last_gradient = gradient_values
-        return gradient_values
+            self.last_gradient = gradient
+        if self.numpy_converter:
+            gradient = self.numpy_converter(gradient)
+        return gradient
 
-    def hessian(self, values) -> np.ndarray:
+    def hessian(self, values) -> znp.array:
         """Calculate the hessian like :py:meth:`~ZfitLoss.hessian`.
 
         Args:
@@ -331,19 +363,20 @@ class LossEval:
         if not self._ignoring_maxiter:
             self.nhess_eval += 1
         params = self.params
-        assign_values(params, values=values)
+        assign_values_func(params, values=values)
         is_nan = False
 
         try:
             hessian = self.hesse_fn(params=params)
-            hessian_values = np.array(run(hessian))
-            _, _, hessian = self.strategy.callback(value=None,
-                                                   gradient=None,
-                                                   hessian=hessian,
-                                                   params=params,
-                                                   loss=self.loss)
+            _, _, hessian = self.strategy.callback(
+                value=None,
+                gradient=None,
+                hessian=hessian,
+                params=params,
+                loss=self.loss,
+            )
         except Exception as error:
-            hessian_values = ["invalid"] * len(params)
+            hessian = ["invalid"] * len(params)
             if isinstance(error, tf.errors.InvalidArgumentError):
                 is_nan = True
             else:
@@ -352,32 +385,35 @@ class LossEval:
         finally:
             if self.do_print:
                 try:
-                    print_params(params, values, loss=-999)
+                    print_params(params, values, loss=None)
                 except:
                     print("Cannot print loss value or gradient values.")
-        check_derivative_none_raise(hessian_values, params)
+        check_derivative_none_raise(hessian, params)
 
-        is_nan = is_nan or np.any(np.isnan(hessian_values))
+        is_nan = is_nan or znp.any(znp.isnan(hessian))
         if is_nan:
             self.nan_counter += 1
             info_values = {
-                'loss': -999,
-                'old_loss': self.last_value,
-                'old_grad': self.last_gradient,
-                'nan_counter': self.nan_counter,
+                "loss": None,
+                "old_loss": self.last_value,
+                "old_grad": self.last_gradient,
+                "nan_counter": self.nan_counter,
             }
 
-            _, _ = self.strategy.minimize_nan(loss=self.loss, params=params,
-                                              values=info_values)
+            _, _ = self.strategy.minimize_nan(
+                loss=self.loss, params=params, values=info_values
+            )
         else:
             self.nan_counter = 0
-            self.last_hessian = hessian_values
-        return hessian_values
+            self.last_hessian = hessian
+        if self.numpy_converter:
+            hessian = self.numpy_converter(hessian)
+        return hessian
 
 
 def print_params(params, values, loss=None):
     table = tt.Texttable()
-    table.header(['Parameter', 'Value'])
+    table.header(["Parameter", "Value"])
 
     for param, value in zip(params, values):
         table.add_row([param.name, value])
@@ -388,7 +424,7 @@ def print_params(params, values, loss=None):
 
 def print_gradient(params, values, gradient, loss=None):
     table = tt.Texttable()
-    table.header(['Parameter', 'Value', 'Gradient'])
+    table.header(["Parameter", "Value", "Gradient"])
     for param, value, grad in zip(params, values, gradient):
         table.add_row([param.name, value, grad])
     if loss is not None:
