@@ -240,7 +240,7 @@ class FFTConvPDFV1(BaseFunctor):
             "nbins_func": nbins_func,
         }
 
-    @z.function(wraps="model")
+    @z.function(wraps="model_convolution")
     def _unnormalized_pdf(self, x):
 
         lower_func, upper_func = self._conv_limits["func"]
@@ -363,7 +363,9 @@ class FFTConvPDFV1(BaseFunctor):
         )
 
         return accept_reject_sample(
-            lambda x: tf.ones(shape=tf.shape(x)[0], dtype=self.dtype),  # use inside?
+            lambda x: tf.ones(
+                shape=tf.shape(x.value())[0], dtype=self.dtype
+            ),  # use inside?
             # all the points are inside
             n=n,
             limits=limits,
