@@ -1,24 +1,30 @@
-#  Copyright (c) 2021 zfit
+#  Copyright (c) 2022 zfit
 
-from typing import Callable, Dict, Iterable, Union
+from __future__ import annotations
+
+from collections.abc import Iterable, Callable
 
 import tensorflow as tf
 
 import zfit.z.numpy as znp
-
-from ..core.basefunc import BaseFunc
+from ..core.basefunc import BaseFuncV1
 from ..core.basemodel import SimpleModelSubclassMixin
 from ..core.dependents import _extract_dependencies
-from ..core.interfaces import ZfitFunc, ZfitModel
+from ..core.interfaces import ZfitFunc
 from ..core.space import supports
 from ..models.basefunctor import FunctorMixin
 from ..util import ztyping
 from ..util.container import convert_to_container
 
 
-class SimpleFunc(BaseFunc):
-
-    def __init__(self, obs: ztyping.ObsTypeInput, func: Callable, name: str = "Function", **params):
+class SimpleFuncV1(BaseFuncV1):
+    def __init__(
+        self,
+        obs: ztyping.ObsTypeInput,
+        func: Callable,
+        name: str = "Function",
+        **params,
+    ):
         """Create a simple function out of of `func` with the observables `obs` depending on `parameters`.
 
         Args:
@@ -37,7 +43,7 @@ class SimpleFunc(BaseFunc):
             return self._value_func(self, x)
 
 
-class BaseFunctorFunc(FunctorMixin, BaseFunc):
+class BaseFunctorFuncV1(FunctorMixin, BaseFuncV1):
     def __init__(self, funcs, name="BaseFunctorFunc", params=None, **kwargs):
         funcs = convert_to_container(funcs)
         if params is None:
@@ -47,19 +53,22 @@ class BaseFunctorFunc(FunctorMixin, BaseFunc):
 
         self.funcs = funcs
         super().__init__(name=name, models=self.funcs, params=params, **kwargs)
+        self._models = self.funcs
 
     def _get_dependencies(self):  # TODO: change recursive to `only_floating`?
         dependents = super()._get_dependencies()  # get the own parameter dependents
         func_dependents = _extract_dependencies(self.funcs)  # flatten
         return dependents.union(func_dependents)
 
-    @property
-    def _models(self) -> Dict[Union[float, int, str], ZfitModel]:
-        return self.funcs
 
-
-class SumFunc(BaseFunctorFunc):
-    def __init__(self, funcs: Iterable[ZfitFunc], obs: ztyping.ObsTypeInput = None, name: str = "SumFunc", **kwargs):
+class SumFunc(BaseFunctorFuncV1):
+    def __init__(
+        self,
+        funcs: Iterable[ZfitFunc],
+        obs: ztyping.ObsTypeInput = None,
+        name: str = "SumFunc",
+        **kwargs,
+    ):
         super().__init__(funcs=funcs, obs=obs, name=name, **kwargs)
 
     def _func(self, x):
@@ -69,14 +78,22 @@ class SumFunc(BaseFunctorFunc):
         return sum_funcs
 
     @supports()
-    def _analytic_integrate(self, limits, norm_range):
+    def _analytic_integrate(self, limits, norm):
         # below may raises AnalyticIntegralNotImplementedError, that's fine. We don't wanna catch that.
-        integrals = [func.analytic_integrate(limits=limits, norm_range=norm_range) for func in self.funcs]
+        integrals = [
+            func.analytic_integrate(limits=limits, norm=norm) for func in self.funcs
+        ]
         return tf.math.accumulate_n(integrals)
 
 
-class ProdFunc(BaseFunctorFunc):
-    def __init__(self, funcs: Iterable[ZfitFunc], obs: ztyping.ObsTypeInput = None, name: str = "SumFunc", **kwargs):
+class ProdFunc(BaseFunctorFuncV1):
+    def __init__(
+        self,
+        funcs: Iterable[ZfitFunc],
+        obs: ztyping.ObsTypeInput = None,
+        name: str = "SumFunc",
+        **kwargs,
+    ):
         super().__init__(funcs=funcs, obs=obs, name=name, **kwargs)
 
     def _func(self, x):
@@ -85,7 +102,7 @@ class ProdFunc(BaseFunctorFunc):
         return product
 
 
-class ZFunc(SimpleModelSubclassMixin, BaseFunc):
+class ZFuncV1(SimpleModelSubclassMixin, BaseFuncV1):
     def __init__(self, obs: ztyping.ObsTypeInput, name: str = "ZFunc", **params):
         super().__init__(obs=obs, name=name, **params)
 
