@@ -3,19 +3,18 @@
 One example is a normal function `Function` that allows to simply define a non-normalizable function.
 """
 
-#  Copyright (c) 2021 zfit
+#  Copyright (c) 2022 zfit
 
 import functools
-from types import MethodType
 
 import tensorflow as tf
 
+from .functor import BaseFunctor
 from ..core.basemodel import SimpleModelSubclassMixin
 from ..core.basepdf import BasePDF
-from ..core.space import no_norm_range
+from ..core.space import supports
 from ..util import ztyping
 from ..util.exception import NormRangeNotImplemented
-from .functor import BaseFunctor
 
 
 class SimplePDF(BasePDF):
@@ -29,26 +28,27 @@ class SimplePDF(BasePDF):
         except TypeError:
             return self._unnormalized_prob_func(self, x)
 
-    def copy(self, **override_parameters) -> 'BasePDF':
+    def copy(self, **override_parameters) -> "BasePDF":
         override_parameters.update(func=self._unnormalized_prob_func)
         return super().copy(**override_parameters)
 
 
 class SimpleFunctorPDF(BaseFunctor, SimplePDF):
-
     def __init__(self, obs, pdfs, func, name="SimpleFunctorPDF", **params):
         super().__init__(obs=obs, pdfs=pdfs, func=func, name=name, **params)
 
 
 def raise_error_if_norm_range(func):
-    func = no_norm_range(func)
+    func = supports(norm=False)(func)
 
     @functools.wraps(func)
     def wrapped(*args, **kwargs):
         try:
             return func(*args, **kwargs)
         except NormRangeNotImplemented:  # TODO: silently remove norm_range? Or loudly fail?
-            raise tf.errors.InvalidArgumentError("Norm_range given to Function: cannot be normalized.")
+            raise tf.errors.InvalidArgumentError(
+                "Norm_range given to Function: cannot be normalized."
+            )
 
     return wrapped
 
