@@ -1,7 +1,7 @@
-#  Copyright (c) 2021 zfit
+#  Copyright (c) 2022 zfit
 
 
 from .minimizers.errors import compute_errors
 from .minimizers.fitresult import Approximations, FitResult
 
-__all__ = ['FitResult', 'compute_errors', 'Approximations']
+__all__ = ["FitResult", "compute_errors", "Approximations"]
